package com.example.xmlparser;

import java.net.URL;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserFactory;

import android.app.ProgressDialog;
import android.os.AsyncTask;

/*
 * Example of a generally useful XML background parser. To use, 
 * instantiate it with a valid, non-null XMLData adapter as input 
 * as well as a completely setup ProgressDialog. Then issue the command 
 * 
 *      "loader.execute(URL-String)" 
 *      
 * with a URL pointing to the XML data to load. This class will:
 * 
 *     - clear the adapter of all data
 *     - show a progress dialog while data is loading
 *     - process the XML input file and extract records
 *     - add the records as objects to the adapter
 *     - notify the adapter when all data is loaded
 *     - dismiss the progress bar
 *     - pass along any error message as new Items
 *     
 * Requirements:
 * 
 *   - the data to be loaded must be of a form where each 'record'
 *     is between <item> and </item> tags. In addition, each field
 *     in the record that is supposed to be loaded must have a corresponding
 *     entry in the Item class, together with appropriate code in the 
 *     "setField" method. For example, to load a list of:
 *     
 *     <item>
 *        <name>John Doe</name>
 *        <phone>800 555 1212</phone>
 *     </item>
 *     ...
 *     
 *     items, your Item class must have two fields of type String and the
 *     "setField" method must set the filed "name" if the input field is "name"
 *     and the field "phone" if the input field is phone.
 *   
 *   - Any error messages are passed on to the user by creating an Item with the
 *     error string as input. Thus, the item class must have a constructor with a 
 *     single String as input. It must also have an empty constructor.
 *     
 *   - Loading takes place in a background thread and the main thread will be
 *     notified if the data is completely loaded, or if an error occurs.
 *     
 * To load data of a different format, adjust the Item class type accordingly. Of course,
 * your adapter right now only shows each object as a String. If you want a more
 * complicated (and satisfying) look, you need to also edit the "getView" method
 * of the adapter class.
 */
public class XMLLoader extends AsyncTask<String, Integer, Integer>
{
	private ProgressDialog progress;
	private XMLData data;
	private String error = "no error";
	private boolean canceled = false;
	
	// Initializing the fields
	public XMLLoader(XMLData data, ProgressDialog progress)
	{
		super();
		this.data = data;
		this.progress = progress;
		this.error = "no error";
		this.canceled = false;
	}
	
	// This method is the main parsing loop. It executes automatically when
	// the "execute" method is called. Trace carefully what is going on here
	// to understand the way this method works.
	@Override
	protected Integer doInBackground(String... urls)
	{
		try
		{
			URL url = new URL(urls[0]);
			XmlPullParserFactory creator = XmlPullParserFactory.newInstance();
			XmlPullParser parser = creator.newPullParser();
			parser.setInput(url.openStream(), null);
			
			XMLItem item = null;
			String currentTag = null;
			int event = parser.getEventType();

			// starting parse loop until we reach the end of the document
			while ((event != XmlPullParser.END_DOCUMENT) && (!canceled))
			{
				if ((event == XmlPullParser.START_TAG) && (!canceled))
				{
					if ((parser.getName().equals("item")) && (!canceled))
					{
						item = new XMLItem();
					}
					else if (!canceled)
					{	
						currentTag = parser.getName();
					}
				}
				else if ((event == XmlPullParser.TEXT)  && (!canceled))
				{
					String text = parser.getText().trim();
					if ((item != null) && (!text.equals(""))  && (!canceled))
					{
						item.setField(currentTag, text);
					}
				}
				else if ((event == XmlPullParser.END_TAG) &&
						 ((parser.getName().equals("item")))  && (!canceled))
				{
				   data.add(item);
				}
			
				event = parser.next();
			}
			
			return data.getCount();
		}
		catch(Exception ex)
		{
			error = ex.toString();
			return -1;
		}
	}

	// Loading can be canceled by setting "canceled" to true. It does not
	// kill the thread, so the thread could continue even if this method is
	// called. However, eventually the main loop will fall through and the
	// thread will be done.
	public void cancelLoading()
	{
		canceled = true;
	}
	
     // Start progress dialog and clear out any left-over items
    protected void onPreExecute()
    {
    	progress.show();
		data.clear();
		canceled = false;
    }
    
    // When thread is done, check on the number of items generated as reported
    // in 'result' and act accordingly. Dismisses the progress dialog, notifies
    // adapter to update its view.
    protected void onPostExecute(Integer result) 
    {
    	if (canceled)
    		data.add(new XMLItem("Loading was canceled ..."));
    	else if (result < 0)
    		data.add(new XMLItem("Sorry, news feed not avilable. Reason: " + error));
    	else if (result == 0)
    		data.add(new XMLItem("No news at the moment."));

    	progress.dismiss();
		data.notifyDataSetChanged();
	}
}
